require 'tempfile'

module TourMigrator
  module Floorplan
    class Report
      attr_reader :svg

      FILE_SIZE_WARN = 300
      FILE_SIZE_ERROR = 500

      def initialize(svg)
        @svg = svg
      end

      def size_in_kilobytes
        return @size_in_kilobytes if defined?(@size_in_kilobytes)

        tempfile = Tempfile.new('floorplan-report')
        tempfile.write(svg.to_xml)
        @size_in_kilobytes = tempfile.size.to_f / 1024
      end

      def to_s
        report_lines = ['<!--']
        report_lines << 'Generated on: ' + Time.now.strftime('%F %H:%M')

        report_lines << '*** INFO ***'

        hotspots = svg.css('.hotspot')
        report_lines << 'Hotspots'
        report_lines << "  count: #{hotspots.count}"
        report_lines << '  linked scene names:'
        hotspots.each_with_index do |hotspot, index|
          report_lines << '    ' + index.to_s + ': ' + hotspot['id'].to_s
        end

        levels = svg.css('.level')
        report_lines << 'Levels'
        report_lines << "  count: #{levels.count}"
        report_lines << '  names:'
        levels.each_with_index do |level, index|
          report_lines << '    ' + index.to_s + ":\t" + level['id'].to_s
        end

        level_switchers = svg.css('.level-switch')
        report_lines << 'Level Switchers'
        report_lines << "  count: #{level_switchers.count}"
        report_lines << '  names:'
        level_switchers.each_with_index do |level_switcher, index|
          report_lines << '    ' + index.to_s + ":\t" + level_switcher['href'].to_s
        end

        report_lines << '*** WARNINGS ***'

        uses = svg.css('use')
        if uses.count.positive?
          report_lines << '  <use> usage:'
          uses.map { |hotspot| hotspot['xlink:href'].to_s.strip }
              .uniq
              .each_with_index do |href, index|
            report_lines << '    ' + index.to_s + ":\txlink:href=" + href
          end
        end

        # TODO: compares scenes.xml with floorplan's hotspots

        links_to_hotspots = hotspots.map { |h| h['id'] }.group_by(&:to_s)
        duplicate_links_to_hotspots = links_to_hotspots.select { |_k, v| v.count > 1 }

        if duplicate_links_to_hotspots.any?
          report_lines << '  hotspots: multiple links to same scene id:'
          duplicate_links_to_hotspots.each do |id, hotspot_ids|
            report_lines << "    scene id: #{id}, link count: #{hotspot_ids.count}"
          end
        end

        images = svg.css('image')
        if images.count.positive?
          report_lines << '  <image> usage:'
          images.each_with_index do |image, index|
            report_lines << '    ' + index.to_s + ":\txlink:href=" + image['xlink:href'].to_s[0..25] + '…'
          end
        end

        report_lines << "file size in kB: #{size_in_kilobytes}" if file_size_warning?(size_in_kilobytes)

        report_lines << '*** ERRORS ***'
        report_lines << '  <style> element found!' if svg.css('style').count.positive?

        # TODO: check unique names for hotspot for each scene

        # hotspot ath and atv is number (int/float),
        # hotspot has linkedscene or exittocitytour

        href_hotspots = svg.css('.hotspot').to_a.keep_if { |h| !!h['xlink:href'] }
        if href_hotspots.count.positive?
          report_lines << '  .hotspot xlink:href:'
          href_hotspots.each_with_index do |hotspot, index|
            report_lines << '    ' + index.to_s + ":\txlink:href=" + hotspot['xlink:href']
          end
        end

        if svg_element = svg.css('svg').first
          if !svg_element['viewBox']
            report_lines << '  svg missing viewBox attribute!'
            report_lines << '  svg has width attribute!' if svg_element['width']
            report_lines << '  svg has height attribute!' if svg_element['height']
          else
            view_box = svg_element['viewBox'].split(' ').map(&:to_i)
            height = view_box[2] - view_box[0]
            width = view_box[3] - view_box[1]

            report_lines << '  svg dimensions larger than 400px!' if width > 400 || height > 400
            report_lines << '  svg dimensions are not square!' if width != height
          end
        end

        report_lines << "  - file size in kB: #{size_in_kilobytes}" if file_size_error?(size_in_kilobytes)

        linked_tours = hotspots.map { |hotspot| hotspot['id'].to_s.split('/').first }
        report_lines << '  - hotspots linking to different tours!' if linked_tours.uniq.count > 1
        report_lines << '-->'

        report_lines.join("\n")
      end

      private

      def file_size_warning?(size_in_kilobytes)
        size_in_kilobytes >= FILE_SIZE_WARN && size_in_kilobytes < FILE_SIZE_ERROR
      end

      def file_size_error?(size_in_kilobytes)
        size_in_kilobytes >= FILE_SIZE_ERROR
      end
    end
  end
end
